# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

from flask.ext.assets import Environment

assets = Environment()


def init_webassets(self):
    """
    Initialize web assets.

    :returns: None

    `webassets <https://github.com/miracle2k/webassets>`_ make it simpler
    to process and bundle CSS and Javascript assets.  This can be baked
    into a Flask application using
    `Flask-Assets <http://flask-assets.readthedocs.org/en/latest/>`_
    """

    assets.init_app(self.app)



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

from flask.ext.sqlalchemy import SQLAlchemy

db = SQLAlchemy()


def init_database(self):
    """
    Initialize database

    :returns: None

    Flask-Diamond assumes you are modelling your solution using an Entity-
    Relationship framework, and that the application will use a relational
    database (e.g. MySQL, Postgres, or SQlite3) for model persistence.
    Thus, `SQLAlchemy
    <http://docs.sqlalchemy.org/en/rel_0_9/contents.html>`_ and `Flask-
    SQLalchemy <https://pythonhosted.org/Flask-SQLAlchemy/index.html>`_
    are used for database operations.

    Typically, this just works as long as ``SQLALCHEMY_DATABASE_URI`` is
    set correctly in the application configuration.
    """

    db.app = self.app
    db.init_app(self.app)



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller


def init_blueprints(self):
    """
    Initialize blueprints.

    :returns: None

    By default, this function does nothing.  Your application needs to
    overload this function in order to implement your View functionality.
    More information about blueprints can be found in the
    `Flask documentation <http://flask.pocoo.org/docs/0.10/blueprints/>`_.
    """

    from ..views.diamond import diamond_blueprint
    self.app.register_blueprint(diamond_blueprint)



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

from wtforms.fields import HiddenField, BooleanField


def init_forms(self):
    """
    WTForms helpers

    :returns: None

    `WTForms <http://wtforms.simplecodes.com/docs/>`_ is a great library
    for using forms and `Flask-WTF <https://flask-
    wtf.readthedocs.org/en/latest/>`_ provides good integration with it.
    WTForms helpers enable you to add custom filters and other custom
    behaviours.
    """

    add_helpers(self.app)


def add_helpers(app):
    """
    Create any Jinja2 helpers needed.
    """

    def is_hidden_field_filter(field):
        return isinstance(field, HiddenField)

    def is_boolean_field_filter(field):
        return isinstance(field, BooleanField)

    app.jinja_env.filters['is_hidden_field'] = is_hidden_field_filter
    app.jinja_env.filters['is_boolean_field'] = is_boolean_field_filter



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

import flask
from builtins import str
from .. import db


class CRUDMixin:
    """
    Convenience functions for CRUD operations.

    Adapted from `flask-kit <https://github.com/semirook/flask-kit/blob/master/base/models.py>`_.
    """

    __table_args__ = {'extend_existing': True}

    @classmethod
    def find(cls, **kwargs):
        """
        Find an object in the database with certain properties.

        :param kwargs: the values of the object to find
        :type kwargs: dict
        :returns: the object that was found, or else None
        """

        return cls.query.filter_by(**kwargs).first()

    @classmethod
    def find_or_create(cls, _commit=True, **kwargs):
        """
        Find an object or, if it does not exist, create it.

        :param kwargs: the values of the object to find or create
        :type kwargs: dict
        :returns: the object that was created
        """

        obj = cls.find(**kwargs)
        if not obj:
            obj = cls.create(_commit=_commit, **kwargs)
        return obj

    @classmethod
    def get_by_id(cls, id):
        """
        Retrieve an object of this class from the database.

        :param id: the id of the object to be retrieved
        :type id: integer
        :returns: the object that was retrieved
        """

        if any(
            (isinstance(id, str) and id.isdigit(),
             isinstance(id, (int, float))),
        ):
            return cls.query.get(int(id))
        return None

    @classmethod
    def create(cls, _commit=True, **kwargs):
        """
        Create a new object.

        :param commit: whether to commit the change immediately to the database
        :type commit: boolean
        :param kwargs: parameters corresponding to the new values
        :type kwargs: dict
        :returns: the object that was created
        """

        instance = cls(**kwargs)
        obj = instance.save(_commit)
        flask.current_app.logger.debug("create %s" % str(obj))
        return obj

    def update(self, _commit=True, **kwargs):
        """
        Update this object with new values.

        :param commit: whether to commit the change immediately to the database
        :type commit: boolean
        :param kwargs: parameters corresponding to the new values
        :type kwargs: dict
        :returns: the object that was updated
        """

        for attr, value in kwargs.items():
            setattr(self, attr, value)
        return _commit and self.save() or self

    def save(self, _commit=True):
        """
        Save this object to the database.

        :param commit: whether to commit the change immediately to the database
        :type commit: boolean
        :returns: the object that was saved
        """

        db.session.add(self)
        if _commit:
            db.session.commit()
        return self

    def delete(self, _commit=True):
        """
        Delete this object.

        :param commit: whether to commit the change immediately to the database
        :type commit: boolean
        :returns: whether the delete was successful
        """

        db.session.delete(self)
        return _commit and db.session.commit()

    def __repr__(self):
        return "<{}(id={})>".format(self.__class__.__name__, self.id)

    def __str__(self):
        return self.__repr__()



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

from .logs import *
from .database import *
from .marshalling import *
from .accounts import *
from .debugger import *
from .webassets import *
from .administration import *
from .email import *
from .task_queue import *
from .forms import *
from .rest import *
from .handlers import *
from .configuration import *
from .blueprints import *
from .signals import *



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

from flask.ext.celery import Celery

celery = Celery()


def init_task_queue(self):
    """
    Initialize celery.
    """

    celery.init_app(self.app)



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

from flask.ext.admin import Admin
import flask.ext.security as security
from flask.ext.admin import BaseView, expose, AdminIndexView
from flask.ext.admin.contrib.sqla import ModelView
from flask.ext.admin.base import MenuLink
from flask.ext.security.utils import encrypt_password
import flask

admin = Admin()


def init_administration(self, index_view=None, user=None, role=None):
    """
    Initialize the Administrative GUI.

    :param index_view: the View that will act as the index page of the admin GUI.
    :type index_view: AdminIndexView
    :returns: None

    The administration GUI is substantially derived from `Flask-Admin
    <http://flask-admin.readthedocs.org/en/latest/>`_. When this function
    is called, it will instantiate blueprints so the application serves
    the admin GUI via the URL http://localhost/admin.

    Typically, you will want to call this function even if you override
    it.  The following example illustrates using super() to invoke this
    administration() function from within your own application.

    >>> admin = super(MyApp, self).administration(
    >>>     index_view=MyApp.modelviews.RedirectView(name="Home")
    >>> )
    """

    admin = Admin(
        name=self.app.config["PROJECT_NAME"],
        base_template='admin/login_base.html',
        index_view=index_view or ForceLoginView(name="Home")
    )

    from .. import db

    if not user or not role:
        raise Exception

    admin.init_app(self.app)

    admin.add_view(UserView(user, db.session, category="Admin"))
    admin.add_view(AdminModelView(role, db.session, category="Admin"))

    return admin


class AuthMixin:
    """
    Require user authentication to be accessible
    """

    def is_accessible(self):
        """
        the View is accessible if the User is authenticated
        """

        return security.current_user.is_authenticated()


class AdminMixin:
    """
    Require admin Role to be accessible
    """

    def is_accessible(self):
        """
        the View is accessible if the User has the Admin Role
        """

        return security.current_user.has_role("Admin")


class AuthView(AuthMixin, BaseView):
    """
    A View that requires authentication
    """
    pass


class AdminView(AdminMixin, BaseView):
    """
    A View that requires the Admin Role
    """
    pass


class AuthModelView(AuthMixin, ModelView):
    """
    A ModelView that requires authentication
    """
    pass


class AdminModelView(AdminMixin, ModelView):
    """
    A ModelView that requires the Admin Role
    """
    pass


class AuthenticatedMenuLink(AuthMixin, MenuLink):
    """
    A MenuLink that requires authentication
    """
    pass


class UserView(AdminModelView):
    """
    Manage the User Model
    """

    column_filters = ['email']
    column_exclude_list = ('password', 'active', 'confirmed_at')
    column_searchable_list = ('email', )
    can_delete = False

    create_template = 'admin/create_user.html'

    def create_model(self, form):
        self.model.register(
            email=form.data["email"],
            password=form.data["password"],
            confirmed=True,
            roles=["User"],
        )
        return flask.redirect(flask.url_for("user.index_view"))

    def update_model(self, form, model):
        original_password = model.password
        model.update(**form.data)
        if form.data["password"] != original_password:
            model.password = encrypt_password(form.data["password"])
            model.save()
        return flask.redirect(flask.url_for("user.index_view"))


class ForceLoginView(AdminIndexView):
    """
    Allocate the root URL and require authentication
    """

    def is_accessible(self):
        """
        the View is accessible if the User is authenticated
        """

        return security.current_user.is_authenticated()

    @expose('/')
    def index(self):
        return self.render("/admin/index.html")



"""Initialize User and Role

Revision ID: cf0f5b45967
Revises: None
Create Date: 2014-03-06 12:55:50.423498

"""

# revision identifiers, used by Alembic.
revision = 'cf0f5b45967'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('role',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=80), nullable=True),
    sa.Column('description', sa.String(length=255), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('name')
    )
    op.create_table('user',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('email', sa.String(length=255), nullable=True),
    sa.Column('password', sa.String(length=255), nullable=False),
    sa.Column('active', sa.Boolean(), nullable=True),
    sa.Column('confirmed_at', sa.DateTime(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('email')
    )
    op.create_table('roles_users',
    sa.Column('user_id', sa.Integer(), nullable=True),
    sa.Column('role_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['role_id'], ['role.id'], ),
    sa.ForeignKeyConstraint(['user_id'], ['user.id'], ),
    sa.PrimaryKeyConstraint()
    )
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('roles_users')
    op.drop_table('user')
    op.drop_table('role')
    ### end Alembic commands ###



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

from flask.ext.marshmallow import Marshmallow

ma = Marshmallow()


def init_marshalling(self):
    """
    Initialize Marshmallow.

    :returns: None
    """

    ma.app = self.app
    ma.init_app(self.app)



# -*- coding: utf-8 -*-

from flask.ext.security import RoleMixin
from .. import db
from .. import ma
from ..mixins.crud import CRUDMixin
from ..mixins.marshmallow import MarshmallowMixin


class RoleSchema(ma.Schema):
    class Meta:
        additional = (
            "id",
            "name",
            "description",
        )


class Role(db.Model, RoleMixin, CRUDMixin, MarshmallowMixin):
    """
    For the purpose of access controls, Roles can be used to create
    collections of users and give them permissions as a group.
    """

    __schema__ = RoleSchema

    id = db.Column(db.Integer(), primary_key=True)
    "integer -- primary key"

    name = db.Column(db.String(80), unique=True)
    "string -- what the role is called"

    description = db.Column(db.String(255))
    "string -- a sentence describing the role"

    def __str__(self):
        return self.name

    @classmethod
    def add_default_roles(cls):
        """
        Create a basic set of users and roles

        :returns: None
        """

        from .. import security

        # make roles
        security.Security.user_datastore.find_or_create_role("Admin")
        security.Security.user_datastore.find_or_create_role("User")
        db.session.commit()



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

from flask.ext.debugtoolbar import DebugToolbarExtension

toolbar = DebugToolbarExtension()


def init_debugger(self):
    """
    Initialize the DebugToolbar

    :returns: None

    The `DebugToolbar <http://flask-
    debugtoolbar.readthedocs.org/en/latest/>`_ is a handy utility for
    debugging your application during development.

    This function obeys the ``DEBUG_TOOLBAR`` configuration setting.  Only
    if this value is explicitly set to True will the Debug Toolbarr run.
    """

    if 'DEBUG_TOOLBAR' in self.app.config and self.app.config['DEBUG_TOOLBAR']:
        toolbar.init_app(self.app)



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller


def init_configuration(self):
    """
    Load the application configuration from the ``SETTINGS`` environment variable.

    :returns: None

    ``SETTINGS`` must contain a filename that points to the configuration file.
    """

    self.app.config.from_envvar('SETTINGS')



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

import re
import os
from setuptools import setup
from distutils.dir_util import copy_tree


# mimicking flask-admin setup.py
# https://github.com/flask-admin/flask-admin/blob/master/setup.py
def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)


def read(fname):
    return open(fpath(fname)).read()


file_text = read(fpath('flask_diamond/__meta__.py'))


def grep(attrname):
    pattern = r"{0}\W*=\W*'([^']+)'".format(attrname)
    strval, = re.findall(pattern, file_text)
    return strval


setup(
    version=grep('__version__'),
    name='Flask-Diamond',
    description="Flask-Diamond is a batteries-included Flask framework.",
    packages=[
        "flask_diamond",
        "flask_diamond.facets",
        "flask_diamond.mixins",
        "flask_diamond.models",
        "flask_diamond.views",
        "flask_diamond.views.diamond",
    ],
    scripts=[
        "bin/flask-diamond",
    ],
    long_description=read('Readme.rst'),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Flask",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Operating System :: MacOS :: MacOS X",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Internet :: WWW/HTTP",
    ],
    include_package_data=True,
    keywords='',
    author=grep('__author__'),
    author_email=grep('__email__'),
    url=grep('__url__'),
    install_requires=read('requirements.txt'),
    license='MIT',
    zip_safe=False,
)

venv_path = os.environ.get("VIRTUAL_ENV")
if venv_path:
    try:
        copy_tree("skels", os.path.join(venv_path, "share/skels"))
    except:
        print("WARN: failed to install skels.")
        print("flask-diamond may not work correctly as a result.")
else:
    print("WARN: did not install skels.  This was not installed in a virtual environment")
    print("flask-diamond may not work correctly as a result.")



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

import logging


def init_logs(self):
    """
    Initialize a log file to collect messages.

    :returns: None

    This file may be written to using

    >>> flask.current_app.logger.info("message")
    """

    handler = logging.FileHandler(self.app.config['LOG'])
    handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)s: %(message)s'))
    self.app.logger.addHandler(handler)
    if self.app.config.get("LOG_LEVEL") == "DEBUG":
        self.app.logger.setLevel(logging.DEBUG)
    elif self.app.config.get("LOG_LEVEL") == "WARN":
        self.app.logger.setLevel(logging.WARN)
    else:
        self.app.logger.setLevel(logging.INFO)
    self.app.logger.info('Startup with log: %s' % self.app.config['LOG'])



# -*- coding: utf-8 -*-
# Flask-Diamond (c) Ian Dennis Miller

import flask

diamond_blueprint = flask.Blueprint(
    'diamond_blueprint',
    __name__,
    static_folder='static',
    template_folder='templates',
    static_url_path='/static/diamond',
)
